# -*- coding: utf-8 -*-
# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import pytest

from rucio.common.utils import execute, PREFERRED_CHECKSUM, set_preferred_checksum
from rucio.rse import rsemanager
from rucio.tests.common import skip_rse_tests_with_accounts, load_test_conf_file
from .rsemgr_api_test import MgrTestCases


@pytest.mark.noparallel(reason='creates and removes a test directory with a fixed name')
@skip_rse_tests_with_accounts
class TestRseSSH(MgrTestCases):
    original_prefchecksum = PREFERRED_CHECKSUM

    @classmethod
    @pytest.fixture(scope='class')
    def setup_rse_and_files(cls, containerized_rses, vo, tmp_path_factory):
        """SSH (RSE/PROTOCOLS): Creating necessary directories and files """

        # Getting info for the test environment
        rses = [rse for rse in containerized_rses if rse[0] == 'SSH1']
        data = load_test_conf_file('rse_repository.json')
        prefix = data['SSH-DISK']['protocols']['supported']['ssh']['prefix']
        sshuser = data['SSH-DISK']['protocols']['supported']['ssh']['extended_attributes']['user']

        if len(rses) == 0:
            rse_name = 'SSH-RSE'
            hostname = data['SSH-RSE']['protocols']['supported']['ssh']['hostname']
        else:
            rse_name = 'SSH1'
            hostname = 'ssh1'
            prefix = '/rucio/'
            sshuser = 'root'

        try:
            os.mkdir(prefix)
        except Exception as e:
            print(e)

        rse_settings, tmpdir, user = cls.setup_common_test_env(rse_name, vo, tmp_path_factory)

        set_preferred_checksum('md5')
        cmd = 'ssh-keygen -R %s' % (hostname)
        execute(cmd)
        cmd = 'ssh-keyscan %s  >> /root/.ssh/known_hosts' % (hostname)
        execute(cmd)

        protocol = rsemanager.create_protocol(rse_settings, 'write')
        protocol.connect()

        os.system('dd if=/dev/urandom of=%s/data.raw bs=1024 count=1024' % prefix)
        for f in MgrTestCases.files_remote:
            path = str(prefix + protocol._get_path('user.%s' % user, f))
            pathdir = os.path.dirname(path)
            cmd = 'ssh %s@%s "mkdir -p %s" && scp %s/data.raw %s@%s:%s' % (sshuser, hostname, str(pathdir), prefix, sshuser, hostname, path)
            execute(cmd)

        for f in MgrTestCases.files_local_and_remote:
            path = str(prefix + protocol._get_path('user.%s' % user, f))
            pathdir = os.path.dirname(path)
            cmd = 'ssh {0}@{1} "mkdir -p {2}" && scp {3}/{4} {5}@{6}:{7}'.format(sshuser, hostname, str(pathdir), str(tmpdir), str(f), sshuser, hostname, path)
            execute(cmd)

        yield rse_settings, tmpdir, user

        clean_raw = '%s/data.raw' % prefix
        list_files_cmd_user = 'ssh %s@%s find %s/user/%s' % (sshuser, hostname, prefix, user)
        clean_files = str(execute(list_files_cmd_user)[1]).split('\n')
        list_files_cmd_group = 'ssh %s@%s find %s/group/%s' % (sshuser, hostname, prefix, user)
        clean_files += str(execute(list_files_cmd_group)[1]).split('\n')
        clean_files.append(clean_raw)
        for files in clean_files:
            clean_cmd = 'ssh %s@%s rm %s' % (sshuser, hostname, files)

        list_directory = 'ssh %s@%s find %s/user/%s' % (sshuser, hostname, prefix, user)
        clean_directory = str(execute(list_directory)[1]).split('\n')
        list_directory_group = 'ssh %s@%s find %s/group/%s' % (sshuser, hostname, prefix, user)
        clean_directory += str(execute(list_directory_group)[1]).split('\n')
        for directory in clean_directory:
            clean_cmd = 'ssh %s@%s rm -r %s' % (sshuser, hostname, directory)
            execute(clean_cmd)

        set_preferred_checksum(cls.original_prefchecksum)
        cmd = 'ssh-keygen -R %s' % (hostname)
        execute(cmd)

    @pytest.fixture(autouse=True)
    def setup_obj(self, setup_rse_and_files, vo):
        rse_settings, tmpdir, user = setup_rse_and_files
        self.init(tmpdir=tmpdir, rse_settings=rse_settings, user=user, vo=vo)

    def test_delete_mgr_ok_dir(self):
        raise pytest.skip("Not implemented")



# -*- coding: utf-8 -*-
# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from json import loads

import pytest

from rucio.common.exception import AccountNotFound, Duplicate, ScopeNotFound, InvalidObject
from rucio.common.types import InternalScope
from rucio.common.utils import generate_uuid as uuid
from rucio.core.scope import get_scopes, add_scope, is_scope_owner
from rucio.tests.common import account_name_generator, scope_name_generator, headers, auth, hdrdict


class TestScopeCoreApi:

    def test_list_scopes(self, vo, jdoe_account):
        scopes = [InternalScope(scope_name_generator(), vo=vo) for _ in range(5)]
        """ SCOPE (CORE): List scopes """
        for scope in scopes:
            add_scope(scope=scope, account=jdoe_account)
        scopes = get_scopes(account=jdoe_account)
        for scope in scopes:
            assert scope in scopes

    def test_is_scope_owner(self, vo, jdoe_account):
        """ SCOPE (CORE): Is scope owner """
        scope = InternalScope(scope_name_generator(), vo=vo)
        add_scope(scope=scope, account=jdoe_account)
        anwser = is_scope_owner(scope=scope, account=jdoe_account)
        assert anwser is True


def test_scope_success(rest_client, auth_token):
    """ SCOPE (REST): send a POST to create a new account and scope """
    acntusr = account_name_generator()
    data = {'type': 'USER', 'email': 'rucio.email.com'}
    response = rest_client.post('/accounts/' + acntusr, headers=headers(auth(auth_token)), json=data)
    assert response.status_code == 201

    scopeusr = scope_name_generator()
    response = rest_client.post('/accounts/%s/scopes/%s' % (acntusr, scopeusr), headers=headers(auth(auth_token)))
    assert response.status_code == 201


def test_scope_failure(rest_client, auth_token):
    """ SCOPE (REST): send a POST to create a new scope for a not existing account to test the error"""
    scopeusr = scope_name_generator()
    account_name_generator()
    response = rest_client.post('/accounts/%s/scopes/%s' % (scopeusr, scopeusr), headers=headers(auth(auth_token)))
    assert response.status_code == 404


def test_scope_duplicate(rest_client, auth_token):
    """ SCOPE (REST): send a POST to create a already existing scope to test the error"""
    acntusr = account_name_generator()
    data = {'type': 'USER', 'email': 'rucio@email.com'}
    response = rest_client.post('/accounts/' + acntusr, headers=headers(auth(auth_token)), json=data)
    assert response.status_code == 201

    scopeusr = scope_name_generator()
    response = rest_client.post('/accounts/%s/scopes/%s' % (acntusr, scopeusr), headers=headers(auth(auth_token)))
    assert response.status_code == 201
    response = rest_client.post('/accounts/%s/scopes/%s' % (acntusr, scopeusr), headers=headers(auth(auth_token)))
    assert response.status_code == 409


def test_list_scope(rest_client, auth_token):
    """ SCOPE (REST): send a GET list all scopes for one account """
    tmp_val = account_name_generator()
    headers_dict = {'Rucio-Type': 'user', 'X-Rucio-Account': 'root'}
    data = {'type': 'USER', 'email': 'rucio@email.com'}
    response = rest_client.post('/accounts/%s' % tmp_val, headers=headers(auth(auth_token), hdrdict(headers_dict)), json=data)
    assert response.status_code == 201

    scopes = [scope_name_generator() for _ in range(5)]
    for scope in scopes:
        response = rest_client.post('/accounts/%s/scopes/%s' % (tmp_val, scope), headers=headers(auth(auth_token)), json={})
        assert response.status_code == 201

    response = rest_client.get('/accounts/%s/scopes/' % tmp_val, headers=headers(auth(auth_token)))
    assert response.status_code == 200

    svr_list = loads(response.get_data(as_text=True))
    for scope in scopes:
        assert scope in svr_list


def test_list_scope_account_not_found(rest_client, auth_token):
    """ SCOPE (REST): send a GET list all scopes for a not existing account """
    response = rest_client.get('/accounts/testaccount/scopes/', headers=headers(auth(auth_token)))
    assert response.status_code == 404
    assert response.headers.get('ExceptionClass') == 'AccountNotFound'


def test_list_scope_no_scopes(rest_client, auth_token):
    """ SCOPE (REST): send a GET list all scopes for one account without scopes """
    acntusr = account_name_generator()
    data = {'type': 'USER', 'email': 'rucio@email.com'}

    response = rest_client.post('/accounts/' + acntusr, headers=headers(auth(auth_token)), json=data)
    assert response.status_code == 201

    response = rest_client.get('/accounts/%s/scopes/' % acntusr, headers=headers(auth(auth_token)))
    assert response.status_code == 404
    assert response.headers.get('ExceptionClass') == 'ScopeNotFound'


class TestScopeClient:

    def test_create_scope(self, rucio_client):
        """ SCOPE (CLIENTS): create a new scope."""
        account = 'jdoe'
        scope = scope_name_generator()
        ret = rucio_client.add_scope(account, scope)
        assert ret
        with pytest.raises(InvalidObject):
            rucio_client.add_scope(account, 'tooooolooooongscooooooooooooope')
        with pytest.raises(InvalidObject):
            rucio_client.add_scope(account, '$?!')

    def test_create_scope_no_account(self, rucio_client):
        """ SCOPE (CLIENTS): try to create scope for not existing account."""
        account = str(uuid()).lower()[:30]
        scope = scope_name_generator()
        with pytest.raises(AccountNotFound):
            rucio_client.add_scope(account, scope)

    def test_create_scope_duplicate(self, rucio_client):
        """ SCOPE (CLIENTS): try to create a duplicate scope."""
        account = 'jdoe'
        scope = scope_name_generator()
        rucio_client.add_scope(account, scope)
        with pytest.raises(Duplicate):
            rucio_client.add_scope(account, scope)

    def test_list_scopes(self, rucio_client):
        """ SCOPE (CLIENTS): try to list scopes for an account."""
        account = 'jdoe'
        scope_list = [scope_name_generator() for _ in range(5)]
        for scope in scope_list:
            rucio_client.add_scope(account, scope)

        svr_list = rucio_client.list_scopes_for_account(account)

        for scope in scope_list:
            if scope not in svr_list:
                assert False

    def test_list_scopes_account_not_found(self, rucio_client):
        """ SCOPE (CLIENTS): try to list scopes for a non existing account."""
        account = account_name_generator()
        with pytest.raises(AccountNotFound):
            rucio_client.list_scopes_for_account(account)

    def test_list_scopes_no_scopes(self, rucio_client):
        """ SCOPE (CLIENTS): try to list scopes for an account without scopes."""
        account = account_name_generator()
        rucio_client.add_account(account, 'USER', 'rucio@email.com')
        with pytest.raises(ScopeNotFound):
            rucio_client.list_scopes_for_account(account)




'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '32.8.0',
    'branch_nick': 'release-32-LTS',
    'revision_id': 'd8ff43713d63f1a6295fe21df8321f62b429a568',
    'revno': 12656
}



# -*- coding: utf-8 -*-
# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Hermes Test
"""

from datetime import datetime
from json import loads
import requests
import pytest

import stomp
import time

from rucio.common.config import config_get, config_get_int
from rucio.core.message import add_message, retrieve_messages, truncate_messages
from rucio.daemons.hermes import hermes
from rucio.tests.common import rse_name_generator, skip_missing_elasticsearch_influxdb_in_env


class MyListener(object):
    def __init__(self, conn):
        self.conn = conn
        self.count = 0
        self.messages = []

    def reset(self):
        self.count = 0
        self.messages = []

    def on_error(self, headers, message):
        print("received an error %s" % message)

    def on_message(self, frame):
        print("received message %s" % frame)
        message = frame.body
        self.count += 1
        self.messages.append(loads(message))


@pytest.mark.noparallel(reason="fails when run in parallel")
@skip_missing_elasticsearch_influxdb_in_env
@pytest.mark.parametrize(
    "core_config_mock",
    [
        {
            "table_content": [
                ("hermes", "services_list", "influx,activemq,elastic,email"),
                (
                    "hermes",
                    "elastic_endpoint",
                    "http://localhost:9200/ddm_events/doc/_bulk",
                ),
                (
                    "hermes",
                    "influxdb_endpoint",
                    "http://localhost:8086/api/v2/write?org=rucio&bucket=rucio",
                ),
                ("hermes", "influxdb_token", "mytoken"),
                ("messaging-hermes", "destination", "/queue/events"),
                ("messaging-hermes", "brokers", "localhost"),
                ("messaging-hermes", "use_ssl", False),
                ("messaging-hermes", "username", "hermes"),
                ("messaging-hermes", "password", "supersecret"),
                ("messaging-hermes", "nonssl_port", 61613),
                ("messaging-hermes", "send_email", False),
            ]
        }
    ],
    indirect=True,
)
@pytest.mark.parametrize(
    "caches_mock",
    [
        {
            "caches_to_mock": [
                "rucio.core.config.REGION",
            ]
        }
    ],
    indirect=True,
)
def test_hermes(core_config_mock, caches_mock):
    """HERMES (DAEMON): Test the messaging daemon."""
    truncate_messages()
    mock_rse = rse_name_generator()
    file_size = 2
    nb_messages = 3
    list_messages = []
    event_types = ["blahblah", "deletion-done"]

    # Start consumer
    host = config_get("messaging-hermes", "brokers")
    port = config_get_int("messaging-hermes", "port")
    user = config_get("messaging-hermes", "username")
    password = config_get("messaging-hermes", "password")
    destination = config_get("messaging-hermes", "destination")
    conn = stomp.Connection(host_and_ports=[(host, port)])
    listener = MyListener(conn)
    conn.set_listener("", listener)
    conn.connect(login=user, passcode=password)
    conn.subscribe(
        destination=destination,
        id=1,
        ack="auto",
        headers={
            "subscription-type": "MULTICAST",
            "durable-subscription-name": "someValue",
        },
    )
    for _ in range(10):
        if conn.is_connected():
            break
        time.sleep(2)
    listener.reset()
    print("Waiting for messages...")

    # Create 3 messages of type blahblah registered to services influx, activemq and elastic
    # Create 3 messages of type email registered to service email
    for i in range(1, 4):
        event_type = event_types[0]
        message = {
            "bytes": 2,
            "rse": mock_rse,
            "created_at": datetime.utcnow().replace(microsecond=0),
        }
        add_message(event_type, message)
        add_message(
            "email",
            {
                "to": config_get("messaging-hermes", "email_test").split(","),
                "subject": "Half-Life %i" % i,
                "body": """
                              Good morning, and welcome to the Black Mesa Transit System.

                              This automated train is provided for the security and convenience of
                              the Black Mesa Research Facility personnel. The time is eight-forty
                              seven A.M... Current outside temperature is ninety three degrees with
                              an estimated high of one hundred and five. Before exiting the train,
                              be sure to check your area for personal belongings.

                              Thank you, and have a very safe, and productive day.""",
            },
        )
        message["event_type"] = event_type
        list_messages.append(message)

    messages = retrieve_messages(50, old_mode=False)
    service_dict = {"influx": 0, "elastic": 0, "email": 0, "activemq": 0}
    for message in messages:
        service_dict[message["services"]] += 1
    assert service_dict["influx"] == 3
    assert service_dict["elastic"] == 3
    assert service_dict["activemq"] == 3
    assert service_dict["email"] == 3

    # Run Hermes
    # The messages of event_type email should be submitted and removed from the list
    # The messages of event-type blahblah should be removed from the list for service influx since this event-type is not supported by influx
    # The messages of event-type blahblah should be submitted to elastic
    # The messages of event-type blahblah should be submitted to ActiveMQ
    hermes.hermes(once=True)
    service_dict = {"influx": 0, "elastic": 0, "email": 0, "activemq": 0}
    messages = retrieve_messages(50, old_mode=False)
    for message in messages:
        service_dict[message["services"]] += 1
    assert service_dict["influx"] == 0
    assert service_dict["elastic"] == 0
    assert service_dict["activemq"] == 0
    assert service_dict["email"] == 0

    # Now add nb_messages more messages of event-type deletion-done associated to services influx, elastic and activemq
    for _ in range(nb_messages):
        event_type = event_types[1]
        message = {
            "bytes": file_size,
            "rse": mock_rse,
            "created_at": datetime.utcnow().replace(microsecond=0),
        }
        add_message(event_type, message)
        message["event_type"] = event_type
        list_messages.append(message)

    messages = retrieve_messages(50, old_mode=False)
    service_dict = {"influx": 0, "elastic": 0, "email": 0, "activemq": 0}
    for message in messages:
        service_dict[message["services"]] += 1
    assert service_dict["influx"] == 3
    assert service_dict["elastic"] == 3
    assert service_dict["activemq"] == 3
    assert service_dict["email"] == 0

    # Run Hermes
    hermes.hermes(once=True)
    service_dict = {"influx": 0, "elastic": 0, "email": 0, "activemq": 0}
    messages = retrieve_messages(50, old_mode=False)
    for message in messages:
        service_dict[message["services"]] += 1
    time.sleep(20)  # Waiting that all the messages are consumed to check ActiveMQ

    # Checking influxDB
    assert service_dict["influx"] == 0
    res = requests.get(
        "http://localhost:8086/query?db=rucio",
        headers={"Authorization": "Token mytoken"},
        params={"q": "SELECT * FROM deletion"},
    )
    assert res.status_code == 200
    assert "results" in res.json()
    influx_res = res.json()["results"]
    assert "series" in influx_res[0]
    columns = influx_res[0]["series"][0]["columns"]
    rse_index = columns.index("rse")
    rse_included = False
    for res in influx_res[0]["series"][0]["values"]:
        if res[rse_index] == mock_rse:
            rse_included = True
            nb_deletion_done = columns.index("nb_deletion_done")
            bytes_deletion_done = columns.index("bytes_deletion_done")
            assert res[nb_deletion_done] == nb_messages
            assert res[bytes_deletion_done] == nb_messages * file_size
    assert rse_included

    # Checking ElasticSearch
    pattern = "%a, %d %b %Y %H:%M:%S %Z"
    assert service_dict["elastic"] == 0
    data = ' { "query": { "match_all": {} } }'
    headers = {"Content-Type": "application/json"}
    response = requests.post(
        "http://localhost:9200/_search?size=1000", data=data, headers=headers
    )
    assert response.status_code == 200
    res = response.json()
    print(res)
    elastic_messages = []
    for entry in res["hits"]["hits"]:
        message = entry["_source"]
        elastic_messages.append(
            {
                "created_at": datetime.strptime(
                    message["payload"]["created_at"], pattern
                ),
                "event_type": message["event_type"],
                "rse": message["payload"]["rse"],
                "bytes": message["payload"]["bytes"],
            }
        )
    for message in list_messages:
        assert message in elastic_messages

    # Checking ActiveMQ
    assert service_dict["activemq"] == 0
    assert len(listener.messages) == len(list_messages)

    activemq_messages = []
    for message in listener.messages:
        message["payload"]["created_at"] = datetime.strptime(
            message["payload"]["created_at"], pattern
        )
        message["payload"]["event_type"] = message["event_type"]
        activemq_messages.append(message["payload"])
    for message in list_messages:
        assert message in activemq_messages

    # Checking email
    assert service_dict["email"] == 0



# -*- coding: utf-8 -*-
# Copyright European Organization for Nuclear Research (CERN) since 2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from rucio.core.rse import update_rse, get_rse


class TestQoS:

    def test_update_and_remove_rse_qos_class(self, rse_factory):
        """ QoS (CORE): Update and remove QoS class for RSE """
        rse_name, rse_id = rse_factory.make_mock_rse()

        update_rse(rse_id, {'qos_class': 'fast_and_expensive'})
        rse = get_rse(rse_id)
        assert rse['qos_class'] == 'fast_and_expensive'

        update_rse(rse_id, {'qos_class': 'slow_but_cheap'})
        rse = get_rse(rse_id)
        assert rse['qos_class'] == 'slow_but_cheap'

        update_rse(rse_id, {'qos_class': None})
        rse = get_rse(rse_id)
        assert rse['qos_class'] is None

    def test_update_and_remove_rse_qos_class_client(self, rse_client, rse_factory):
        """ QoS (CLIENT): Update and remove QoS class for RSE """
        rse_name, rse_id = rse_factory.make_mock_rse()

        rse_client.update_rse(rse_name, {'qos_class': 'fast_and_expensive'})
        rse = rse_client.get_rse(rse_name)
        assert rse['qos_class'] == 'fast_and_expensive'

        rse_client.update_rse(rse_name, {'qos_class': 'slow_but_cheap'})
        rse = rse_client.get_rse(rse_name)
        assert rse['qos_class'] == 'slow_but_cheap'

        rse_client.update_rse(rse_name, {'qos_class': None})
        rse = rse_client.get_rse(rse_name)
        assert rse['qos_class'] is None

    def test_qos_policies(self, rse_client, rse_factory):
        """ QoS (CLIENT): Add QoS policy for RSE """
        rse_name, rse_id = rse_factory.make_mock_rse()

        rse_client.add_qos_policy(rse_name, 'FOO')
        policies = rse_client.list_qos_policies(rse_name)
        assert policies == ['FOO']

        rse_client.add_qos_policy(rse_name, 'BAR')
        policies = sorted(rse_client.list_qos_policies(rse_name))
        assert policies == ['BAR', 'FOO']

        rse_client.delete_qos_policy(rse_name, 'BAR')
        policies = rse_client.list_qos_policies(rse_name)
        assert policies == ['FOO']

        rse_client.delete_qos_policy(rse_name, 'FOO')
        policies = rse_client.list_qos_policies(rse_name)
        assert policies == []
